//+------------------------------------------------------------------+
//|                                                      MACD T3.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers   4
#property indicator_color1    LimeGreen
#property indicator_color2    Red
#property indicator_color3    DimGray
#property indicator_color4    Gold
#property indicator_width1    3
#property indicator_width2    3
#property indicator_width3    3

//
//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    FastPeriod      = 12;
extern int    SlowPeriod      = 26;
extern int    SignalPeriod    =  9;
extern int    AppliedPrice    = PRICE_CLOSE;
extern double T3Hot           = 0.618;
extern bool   T3Original      = false;

//
//
//
//
//

double macd[];
double macdhU[];
double macdhD[];
double signal[];
double trend[];

//
//
//
//
//

double emas[][18];
double alphaFast;
double alphaSlow;
double alphaSignal;
double c1;
double c2;
double c3;
double c4;
string indicatorFileName;
bool   returnBars;
int    timeFrame;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0, macdhU);  SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, macdhD);  SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2, macd);
   SetIndexBuffer(3, signal);
   SetIndexBuffer(4, trend);
         timeFrame         = stringToTimeFrame(TimeFrame);
         indicatorFileName = WindowExpertName();
         returnBars        = (TimeFrame=="returnBars"); if (returnBars) return(0);

   //
   //
   //
   //
   //
   
      double a  = T3Hot;
             c1 = -a*a*a;
             c2 =  3*(a*a+a*a*a);
             c3 = -3*(2*a*a+a+a*a*a);
             c4 = 1+3*a+a*a*a+3*a*a;

   //
   //
   //
   //
   //
   
      FastPeriod   = MathMax(1,FastPeriod);
      SlowPeriod   = MathMax(1,SlowPeriod);
      SignalPeriod = MathMax(1,SignalPeriod);
      if (T3Original)
         {
            alphaFast   = 2.0/(1.0 + FastPeriod);
            alphaSlow   = 2.0/(1.0 + SlowPeriod);
            alphaSignal = 2.0/(1.0 + SignalPeriod);
         }            
      else
         {
            alphaFast   = 2.0/(2.0 + (FastPeriod-1.0)/2.0);
            alphaSlow   = 2.0/(2.0 + (SlowPeriod-1.0)/2.0);
            alphaSignal = 2.0/(2.0 + (SignalPeriod-1.0)/2.0);
         }            

   //
   //
   //    
   //
   //

   IndicatorShortName(timeFrameToString(timeFrame)+" MACD T3 ("+FastPeriod+","+SlowPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}  

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { macdhU[0] = limit+1; return(0); }
           if (timeFrame!=Period())
           {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for(int i=limit;i>=0;i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);
                     macdhU[i] = iCustom(NULL,timeFrame,indicatorFileName,"",FastPeriod,SlowPeriod,SignalPeriod,AppliedPrice,T3Hot,T3Original,0,y);
                     macdhD[i] = iCustom(NULL,timeFrame,indicatorFileName,"",FastPeriod,SlowPeriod,SignalPeriod,AppliedPrice,T3Hot,T3Original,1,y);
                     macd[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",FastPeriod,SlowPeriod,SignalPeriod,AppliedPrice,T3Hot,T3Original,2,y);
                     signal[i] = iCustom(NULL,timeFrame,indicatorFileName,"",FastPeriod,SlowPeriod,SignalPeriod,AppliedPrice,T3Hot,T3Original,3,y);
               }
               return(0);
            }                                 
            if (ArrayRange(emas,0) != Bars) ArrayResize(emas,Bars);

   //
   //
   //
   //
   //

   for(i=limit; i>=0; i--) 
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,AppliedPrice,i);
         macd[i]   = iT3(price,alphaFast,0,i)-iT3(price,alphaSlow,6,i);
         signal[i] = iT3(macd[i],alphaSignal,12,i);
         trend[i]  = trend[i+1];
         macdhU[i] = EMPTY_VALUE;
         macdhD[i] = EMPTY_VALUE;
         
         if (macd[i]>macd[i+1]) trend[i] =  1;
         if (macd[i]<macd[i+1]) trend[i] = -1;
            if (trend[i]== 1) macdhU[i] = macd[i];
            if (trend[i]==-1) macdhD[i] = macd[i];
   }

   //
   //
   //
   //
   //
      
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iT3(double price,double alpha,int buffer,int shift)
{
   int i = Bars-shift-1;
   if (i < 1)
      {
         emas[i][0+buffer] = price;
         emas[i][1+buffer] = price;
         emas[i][2+buffer] = price;
         emas[i][3+buffer] = price;
         emas[i][4+buffer] = price;
         emas[i][5+buffer] = price;
      }
   else
      {
         emas[i][0+buffer] = emas[i-1][0+buffer]+alpha*(price            -emas[i-1][0+buffer]);
         emas[i][1+buffer] = emas[i-1][1+buffer]+alpha*(emas[i][0+buffer]-emas[i-1][1+buffer]);
         emas[i][2+buffer] = emas[i-1][2+buffer]+alpha*(emas[i][1+buffer]-emas[i-1][2+buffer]);
         emas[i][3+buffer] = emas[i-1][3+buffer]+alpha*(emas[i][2+buffer]-emas[i-1][3+buffer]);
         emas[i][4+buffer] = emas[i-1][4+buffer]+alpha*(emas[i][3+buffer]-emas[i-1][4+buffer]);
         emas[i][5+buffer] = emas[i-1][5+buffer]+alpha*(emas[i][4+buffer]-emas[i-1][5+buffer]);
      }
   return(c1*emas[i][5+buffer] + c2*emas[i][4+buffer] + c3*emas[i][3+buffer] + c4*emas[i][2+buffer]);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs) {
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
            
            //
            //
            //    check if the no standar time frame exists
            //
            //
            
            int test = StringToInteger(tfs);
               if (test != 0 && iClose(NULL,test,0)!=0) return(test);
                                                        return(Period());
}
string timeFrameToString(int tf) {
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
         if (iClose(NULL,tf,0)!=0) return(""+tf);
                                   return("");
}

//
//
//
//
//

string stringUpperCase(string str) {
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--) {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}